#include <digitalWriteFast.h>  // library for high performance reads and writes by jrraines//

#define c_LeftEncoderInterrupt 0
#define c_LeftEncoderPinA 2
#define c_LeftEncoderPinB 3

/* Define shift register pins used for seven segment display */
#define LATCH_DIO 4
#define CLK_DIO 7
#define DATA_DIO 8

int sensorValue = 0;         // the sensor value
int sensorMin = 1023;        // minimum sensor value
int sensorMax = 0;           // maximum sensor value
int SegNum;
int buttonstate1 = 0;
int buttonstate2 = 0;

const int button1 = 5;
const int button2 = 6;

/* Segment byte maps for numbers 0 to 9 */
const byte SEGMENT_MAP[] = {0xC0,0xF9,0xA4,0xB0,0x99,0x92,0x82,0xF8,0X80,0X90};
/* Byte maps to select digit 1 to 4 */
const byte SEGMENT_SELECT[] = {0xF1,0xF2,0xF4,0xF8};

// These constants won't change:
const int sensorPin = A0;    // pin that the sensor is attached to
const int ledPin = 10;        // pin that the LED is attached to

volatile bool _LeftEncoderBSet;
volatile long _LeftEncoderTicks = 0;

void setup()
{
 // Serial.begin(9600); // used for debug

  pinMode(button1, INPUT);
  pinMode(button2, INPUT);
 
  pinMode(c_LeftEncoderPinA, INPUT);      
  digitalWrite(c_LeftEncoderPinA, LOW); 
  pinMode(c_LeftEncoderPinB, INPUT);
  digitalWrite(c_LeftEncoderPinB, LOW);
  attachInterrupt(c_LeftEncoderInterrupt, HandleLeftMotorInterruptA, RISING);
  /* Set DIO pins to outputs */
  pinMode(LATCH_DIO,OUTPUT);
  pinMode(CLK_DIO,OUTPUT);
  pinMode(DATA_DIO,OUTPUT);

}

// Interrupt service routines for the left motor's quadrature encoder
void HandleLeftMotorInterruptA()
{
  _LeftEncoderBSet = digitalReadFast2(c_LeftEncoderPinB);   // read the input pin
  _LeftEncoderTicks -= _LeftEncoderBSet ? -1 : +1;
}

void loop()
{  
    sensorValue = _LeftEncoderTicks;

    buttonstate1 = digitalRead(button1);
    buttonstate2 = digitalRead(button2);

    if (buttonstate1 == HIGH)
    {
      _LeftEncoderTicks = 0;
      sensorMin = sensorValue;
      }
    if (buttonstate2 == HIGH)
    {
      sensorMax = sensorValue;
      }

    

  /* Update the display with the current counter value */
  if (_LeftEncoderTicks < 0)
  {
    SegNum = 0000;      
    WriteNumberToSegment(0 , SegNum / 1000);
    WriteNumberToSegment(1 , (SegNum / 100) % 10);
    WriteNumberToSegment(2 , (SegNum / 10) % 10);
    WriteNumberToSegment(3 , SegNum % 10);
  }
  else
    {
    WriteNumberToSegment(0 , _LeftEncoderTicks / 1000);
    WriteNumberToSegment(1 , (_LeftEncoderTicks / 100) % 10);
    WriteNumberToSegment(2 , (_LeftEncoderTicks / 10) % 10);
    WriteNumberToSegment(3 , _LeftEncoderTicks % 10);
        // read the sensor:
        
  sensorValue = _LeftEncoderTicks;

  // apply the calibration to the sensor reading
  sensorValue = map(sensorValue, sensorMin, sensorMax, 0, 255);

  // in case the sensor value is outside the range seen during calibration
  sensorValue = constrain(sensorValue, 0, 255);

  // fade the LED using the calibrated value:
  analogWrite(ledPin, sensorValue);
  
  Serial.println(sensorValue);
  Serial.println(_LeftEncoderTicks);
     }

}
/* Write a decimal number between 0 and 9 to one of the 4 digits of the display */
void WriteNumberToSegment(byte Segment, byte Value)
{
digitalWrite(LATCH_DIO,LOW);
shiftOut(DATA_DIO, CLK_DIO, MSBFIRST, SEGMENT_MAP[Value]);
shiftOut(DATA_DIO, CLK_DIO, MSBFIRST, SEGMENT_SELECT[Segment] );
digitalWrite(LATCH_DIO,HIGH);

}
